"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.plugin = void 0;
const graphql_1 = require("graphql");
const getUserDefinedTypes = (schema) => {
    const typeMap = schema.getTypeMap();
    const allTypeNames = Object.keys(typeMap);
    const userDefinedTypes = allTypeNames.filter((typeName) => {
        const type = typeMap[typeName];
        if (!type)
            return;
        return (!(0, graphql_1.isScalarType)(type) &&
            !(0, graphql_1.isSpecifiedScalarType)(type) &&
            !(0, graphql_1.isIntrospectionType)(type));
    });
    return { userDefinedTypes };
};
const plugin = (schema) => {
    const { userDefinedTypes } = getUserDefinedTypes(schema);
    const unionTypeName = userDefinedTypes.reduce((acc, typeName) => {
        if (acc === "")
            return `'${typeName}'`;
        return `${acc} | '${typeName}'`;
    }, "");
    const result = userDefinedTypes
        .map((typeName) => {
        return `export const is${typeName} = (field: { __typename?: ${unionTypeName}; }): field is ${typeName} => field.__typename === '${typeName}';`;
    })
        .filter(Boolean)
        .join("\n");
    return result;
};
exports.plugin = plugin;
